@warnings
@name Fiber FX Estimator

//Version 1.0
//by Steven Pettit 04-Jan 2019
//Program estimates density, cluster radius, and cluster fibers
//And reports back fiber length at 100% scale.


//TODO :  Use a multiselect box for the surfaces, in case more than one
//        surface gets FiberFX applied to it in the same layer.
//TODO :  Dump information to a text file, so it can be printed/used.

var c1,c2,c3,c4,c5,c6;
var outt = @"No Data","-","-","-"@;
var fiblength, fibwidth, fibcnt, srfidx, srf_comb, method;

main {
	
	selmode(USER);
	
	fibwidth = 0.000250;
	fibcnt = 1500000;
		
	//Need this to get surface names
	var t_mesh = Mesh();
	srf_comb = Surface(t_mesh);
	srfidx = 1;
	
	fiblength = getFFXlength();
	
	reqbegin("FiberFX Estimater");
		c1 = ctldistance("Fiber Diameter :",fibwidth);
		c2 = ctlnumber("Max Fibers :", fibcnt);
		c3 = ctlpopup("FiberFX surface:",srfidx,srf_comb);
		c6 = ctlpopup("Cluster Distribution",1,@"powers of 2","Squares","Hexagonal","Fibonacci"@);
		c4 = ctlbutton("Calculate",120, "domagic");
		c5 = ctltext("Results", outt);
		reqpost();
	
		fibwidth = getvalue(c1);
		fibcnt = getvalue(c2);
		srfidx = getvalue(c3);
		method = getvalue(c6);
	
	reqend();
}

domagic
{
	/*
	-Given the mesh, compute the fiber length for fiber scale = 100%.
	
	-Given the fiber diamter, compute the total surface area of the surface
	to have FiberFX applied to it and the diameter of the fiber.  Divide surface
	area by fiber area to get maximum fiber count.
	
	-Given the fibercount for the mesh, using the chosen method, find the density
	that is closest to 100%.
	Methods : Powers of 2 - self explanatory.  f(x) = 2 ^ x (1,2,4,8,16,...)
	          Squares :  f(x) = (2x + 1) ^2 (1,9,25,49,81,...)
			  Hexagonal : Hexagonal Distribution  (1,7,19,37,51...)
			  Fibonacci : Fibonacci sequence (1,1,2,3,5,8,13,21,34...)
			  
	Cluster count is always an integer, while Density is floating point.
	
	*/

	var hcr = -1.0;	
	var tsa = 0.000;
	
	fibwidth = getvalue(c1);
	fibcnt = getvalue(c2);
	srfidx = getvalue(c3);
	method = getvalue(c6);
	
	selpolygon(CLEAR);
	selpolygon(SET,SURFACE,srf_comb[srfidx]);
	selhide(UNSELECTED);
	
	editbegin();
		pcnt = polygons.count();
		moninit(pcnt,"Analyzing Model...");
		foreach(p,polygons){
			//Get centerpoint
			pp = polypointcount(p);
			pa = nil;
			cp = <0,0,0>;
			for(i=1;i<=pp;i++){
				qp = p.points[i];
				qv = pointinfo(qp);
				pa += qv;
				cp += qv / pp;
			}
			pa += pa[1];
			//Get distance to each vertex from center
			for(i=1;i<=pp;i++){
				d = vmag(pa[i] - cp);
				hcr = max(hcr,d);
			}
			tsa += area_poly2(pp,pa, polynormal(p));
			monstep(1);
		}
		monend();
	editend(ABORT);
	//Take hcr, multiply by 1K, and then multiply by sqrt(2)...
	//or, multiply by sqrt(2) * 1000, then round to 2 places...
	rcr = round(hcr * 707.10678,2);
	fibarea = (fibwidth * 2) ^^ 2;
	
	//surface area, in m^2 divided by fiber area, in m^2 = max number of fibers
	numfibers = ceil((tsa / fibarea) / 100) * 100;
	//Let's find a good balance between density and cluster count...
	recd = 0;
	recc = 0;
	//FiberFX eats RAM for really high fiber counts...
	//1.2M fibers tends to give good coverage on large characters
	//While You can come up with working numbers for 24M fibers,
	//Do you have the RAM??
		
	numfibers = min(fibcnt,numfibers);
	
	for(i = 0; i < 11; i++){
		
		switch(method)
		{
			case 1:
				te = powers(i);
				break;
			case 2:
				te = squares(i);
				break;
			case 3:
				te = hexagonal(i);
				break;
			case 4:
				te = fib2(i);
		}		
		
		tc = te * 100000;
		td = numfibers / tc;
		if (td <= 1){
			recd = ceil(100 * td);
			recc = te;
			break;
		}
	}
	
	t1 = sqrt(tsa) * 1000;  // Need millimeters
	t2 = sqrt(numfibers);
	t3 = 2 * (t1/t2);
	rcr += t3;
	rcr /= 2.0;

	outt[1] = "Fiber Length   = " + (fiblength * 1000) + " mm.";
	outt[2] = "Density        = " + recd + " %";
	outt[3] = "Cluster Fibers = " + recc + " fibers.";
	outt[4] = "Cluster Radius = " + rcr +  " mm.";
		
	selunhide();
	selpolygon(CLEAR);
	setvalue(c5,outt);
	requpdate();
}

area_poly2 : q_qty, q_verts, q_norm
{
	var retv = 0;
	var area = 0.0;
	var accum = <0,0,0>;
	for(i=1; i<= q_qty; i++){
		accum += cross3d(q_verts[i], q_verts[i+1]);
	}
	accum /= <2,2,2>;
	
	area = dot3d(q_norm, accum);
	
	retv = area;
	return(retv);
}

getFFXlength
{
	var a,b,c,d;
	var tb = boundingbox();
	var ub = tb[2]-tb[1];
	
	a = ub.x^^2 + ub.y^^2;
	b = ub.y^^2 + ub.z^^2;
	c = ub.z^^2 + ub.x^^2;
	d = max(a, max(b,c));
	d = sqrt(d) / 20.0;
	
	return(d);
}

powers : q
{
	retv = 2 ^^ q;
	return(retv);
}

squares : q
{
	retv = ((2 * q) + 1) ^^ 2;
	return(retv);
}

hexagonal : q
{
	retv = 1;
	i = 1;
	for(i = q; i > 0; i++){
		retv += 6 * i;
	}
	return(retv);
}

fib2 : q
{
	q +=2;
	a = 1.618033988749895 ^^ q;
	a /= sqrt(5);
	a = round(a,0);
	return(a);
}

